/*

OLSC AppDlg.cpp : implementation file

License:

Copyright (c) 2009 Chris Favreau

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

*/

#include "stdafx.h"
#include <direct.h>					// _getcwd()
#include "OLSC App.h"
#include "OLSC AppDlg.h"
#include "ilda.h"					// Test Frame saved as .H file

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// COLSCAppDlg dialog


COLSCAppDlg::COLSCAppDlg(CWnd* pParent /*=NULL*/)
	: CDialog(COLSCAppDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COLSCAppDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CONTROLLERS_COMBO, m_cmbControllers);
}

BEGIN_MESSAGE_MAP(COLSCAppDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_TEST_BUTTON, &COLSCAppDlg::OnBnClickedTestButton)
END_MESSAGE_MAP()


// COLSCAppDlg message handlers

BOOL COLSCAppDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// Fill out the list of available controllers
	FindControllers();

	// All Done!

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void COLSCAppDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR COLSCAppDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

CString GetDirectory(CString *path)
{
	CString result = "";

	int i = path->GetLength()-1;
	
	// Check to see if the path name length is bigger than 1
	if (i < 2) return result;
	
	while(path->GetAt(i) != '\\' && i >= 0)
		i--;
	CString filename = (char *)(path->GetBuffer(1)+i+1);
	result = *path;
	result.TrimRight(filename);
	return result;
}

CString GetFileName(CString *path)
{
	CString result = "";
	int i = path->GetLength()-1;

	// Check to see if the path name length is bigger than 1
	if (i < 2) return result;

	while(path->GetAt(i) != '\\' && i >= 0)
		i--;
	CString filename = (char *)(path->GetBuffer(1)+i+1);
	result = filename;
	return result;
}

void COLSCAppDlg::FindControllers(void)
{
	// Reset our controller list and combo box
	m_cmbControllers.ResetContent();
	for (int i = 0; i < MAX_CONTROLLERS; i++)
	{
		m_olsc[i].UnloadInterface();
	}

	// Get the application's folder
	char app_path[MAX_PATH];
	_getcwd(app_path, _MAX_PATH);
	
	// Append the OLSC folder to the apps folder
	CString szPath;
	szPath.Format("%s\\olsc\\", app_path);

	// Get a list of all the .DLL (and in the future .OLSC files)
	HANDLE hDir;
	WIN32_FIND_DATA tempFindData;
	CString szFilter;
	int iController = 0;

	szFilter.Format("%s%s", szPath, "*.dll");

	hDir = FindFirstFile(szFilter, &tempFindData);

	while (hDir != INVALID_HANDLE_VALUE)
	{
		CString szTemp;
		szTemp.Format("%s%s", szPath, tempFindData.cFileName);
		
		// Load the Controller Library
		if (m_olsc[iController].LoadInterface(szTemp.GetBuffer(MAX_PATH), false))
		{
			// Add the file to the selected files list
			//int index = m_cmbControllers.AddString(GetFileName(&szTemp));
			m_olsc[iController].GetInterfaceName(szTemp.GetBuffer(1024));
			int index = m_cmbControllers.AddString(szTemp);

			// Set the Item Data to the index of the controller we loaded
			m_cmbControllers.SetItemData(index, iController);

			// Next Controller
			iController++;
		}

		// Next File
		if (FindNextFile(hDir, &tempFindData) == 0) break;
	}

	FindClose(hDir);

	// Select the first controller
	m_cmbControllers.SetCurSel(0);
}


void COLSCAppDlg::OnBnClickedTestButton()
{
	// Get the OLSC index of the currently selected controller library
	int iController = (int)m_cmbControllers.GetItemData(m_cmbControllers.GetCurSel());
	
	// Make sure the controller index is good
	if (iController < 0) return;
	if (iController > MAX_CONTROLLERS) return;

	// Check to see if the interface is loaded
	if (!m_olsc[iController].IsInterfaceLoaded()) return;

	// TODO - move this someplace else -- like a button that says "Initialize"
	// Initialize the device
	if (m_olsc[iController].Initialize() < 1) return;

	// TODO - fill out the number of devices in a combo box so the user can select a device

	// Only use device 0

	// Create an OLSC frame
	LASER_SHOW_DEVICE_FRAME frame;
	frame.points = new LASER_SHOW_DEVICE_POINT[test_frame_size];

	// Create a frame of random points
	int i;
	for (i = 0; i < test_frame_size; i++)
	{
		// TODO - for some reason we have to invert the X axis (is this from NLS?)
		// TODO - check the validity of the axis values for X and Y
		frame.points[i].x = 32768 + (test_frame[i][0]);
		frame.points[i].y = 32768 + (test_frame[i][1]);
		frame.points[i].r = test_frame[i][2];
		frame.points[i].g = test_frame[i][3];
		frame.points[i].b = test_frame[i][4];
		frame.points[i].i = 0;
	}
	frame.point_count = test_frame_size;
	frame.display_speed = 30000;

	// Send our test frame to the device
	m_olsc[iController].WriteFrame(0, frame);

	// Play
	m_olsc[iController].Play(0);

	// Turn the beam ON
	m_olsc[iController].Shutter(0, LASER_SHOW_DEVICE_SHUTTER_STATE_ON);

	// DEBUG
	// Wait for a certain amount of time
	//Sleep(5000);

	// Stop
	m_olsc[iController].Pause(0);

	// Turn the beam OFF (using the shutter)

	// TODO - what to do here??? Maybe add start and stop interface things

	// Shutdown the OLSC interface
	//m_olsc[iController].Shutdown();

	// TODO - what else is there to do?

	// Free up our points in the frame
	delete [] frame.points;
}
