/*

*************************************************************
** Open Laser Show Controller Device Structures
**
*************************************************************

License:

Copyright (c) 2009 Chris Favreau

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

*/


#ifndef __LASER_SHOW_DEVICE_STRUCTURES_H__
#define __LASER_SHOW_DEVICE_STRUCTURES_H__

// Constants

// Shutter State
#define LASER_SHOW_DEVICE_SHUTTER_STATE_ON		0
#define LASER_SHOW_DEVICE_SHUTTER_STATE_OFF		1

// Standard Errors
// TODO - add errors to this list so we can be more verbose in describing the particular error that occurred.
#define OLSC_ERROR_SUCCESS						1
#define OLSC_ERROR_NONE							0
#define OLSC_ERROR_INVALID_PARAMETER			-1
#define OLSC_ERROR_FAILED						-2

// Data Structures

struct LASER_SHOW_DEVICE_CAPABILITIES
{
	// Device Name
	char name[1024];
	// Device Version
	int	version_major;
	int	version_minor;
	// Min Speed/Max Speed
	int min_speed;
	int max_speed;
	// Min Frame Size/Max Frame Size
	int min_frame_size;
	int max_frame_size;
	// DMX In/Out?
	bool has_dmx_in;
	bool has_dmx_out;
	// TTL In/Out?
	bool has_ttl_in;
	bool has_ttl_out;
	// X/Y Resolution (8 to 16 bits)
	int	xy_resolution;
	// Color Resolution (1 to 16 bits)
	int color_resolution;
	// Uses callbacks?
	bool uses_callbacks;
};

struct LASER_SHOW_DEVICE_POINT
{
	WORD x;
	WORD y;
	WORD r;
	WORD g;
	WORD b;
	WORD i;
};

struct LASER_SHOW_DEVICE_FRAME
{
	int display_speed;
	int point_count;
	struct LASER_SHOW_DEVICE_POINT *points;
};

#endif
